@@ -17,10 +17,10 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *
- *  $Id: proxy-client.c,v 1.9 2005/01/20 20:56:11 mschimek Exp $
+ *  $Id: proxy-client.c,v 1.12 2006/05/31 03:54:28 mschimek Exp $
  */
 
-static const char rcsid[] = "$Id: proxy-client.c,v 1.9 2005/01/20 20:56:11 mschimek Exp $";
+static const char rcsid[] = "$Id: proxy-client.c,v 1.12 2006/05/31 03:54:28 mschimek Exp $";
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -149,9 +149,9 @@ static vbi_bool proxy_client_connect_server( vbi_proxy_client * vpc )
    {
       dprintf1("connect_server: hostname or port not configured\n");
       if (use_tcp_ip && (vpc->p_srv_host == NULL))
-         vbi_asprintf(&vpc->p_errorstr, _("Server hostname not configured."));
+	 asprintf(&vpc->p_errorstr, _("Server hostname not configured."));
       else if (vpc->p_srv_port == NULL)
-         vbi_asprintf(&vpc->p_errorstr, _("Server port not configured."));
+         asprintf(&vpc->p_errorstr, _("Server port not configured."));
    }
    return result;
 }
@@ -163,7 +163,7 @@ static vbi_bool proxy_client_connect_server( vbi_proxy_client * vpc )
 static vbi_bool proxy_client_alloc_msg_buf( vbi_proxy_client * vpc )
 {
    vbi_bool result;
-   int  msg_size;
+   size_t msg_size;
 
    msg_size = sizeof(VBIPROXY_MSG_BODY);
 
@@ -184,18 +184,20 @@ static vbi_bool proxy_client_alloc_msg_buf( vbi_proxy_client * vpc )
 
    msg_size += VBIPROXY_MSG_BODY_OFFSET;
 
-   if ((msg_size != vpc->max_client_msg_size) || (vpc->p_client_msg == NULL))
+   if (((int) msg_size != vpc->max_client_msg_size)
+       || (vpc->p_client_msg == NULL))
    {
       if (vpc->p_client_msg != NULL)
          free(vpc->p_client_msg);
 
-      dprintf2("alloc_msg_buf: allocate buffer for max. %d bytes\n", msg_size);
+      dprintf2("alloc_msg_buf: allocate buffer for "
+	       "max. %lu bytes\n", (unsigned long) msg_size);
       vpc->max_client_msg_size = msg_size;
       vpc->p_client_msg = malloc(msg_size);
 
       if (vpc->p_client_msg == NULL)
       {
-         vbi_asprintf(&vpc->p_errorstr, _("Virtual memory exhausted."));
+         asprintf(&vpc->p_errorstr, _("Virtual memory exhausted."));
          result = FALSE;
       }
       else
@@ -339,7 +341,7 @@ static vbi_bool proxy_client_take_message( vbi_proxy_client * vpc )
          if (vpc->state == CLNT_STATE_CAPTURING)
          {
             /* XXX TODO check raw */
-            if (pMsg->sliced_ind.sliced_lines > vpc->dec.count[0] + vpc->dec.count[1])
+            if ((int) pMsg->sliced_ind.sliced_lines > vpc->dec.count[0] + vpc->dec.count[1])
             {  /* more lines than req. for service -> would overflow the allocated slicer buffer
                ** -> discard extra lines (should never happen; proxy checks for line counts) */
                dprintf1("take_message: SLICED_IND: too many lines: %d > %d\n", pMsg->sliced_ind.sliced_lines, vpc->dec.count[0] + vpc->dec.count[1]);
@@ -418,7 +420,7 @@ static vbi_bool proxy_client_take_message( vbi_proxy_client * vpc )
    if ((result == FALSE) && (vpc->p_errorstr == NULL))
    {
       dprintf1("take_message: message type %d (len %d) not expected in state %d\n", vpc->p_client_msg->head.type, vpc->p_client_msg->head.len, vpc->state);
-      vbi_asprintf(&vpc->p_errorstr, _("Protocol error (unexpected message)."));
+      asprintf(&vpc->p_errorstr, _("Protocol error (unexpected message)."));
    }
 
    return result;
@@ -579,7 +581,7 @@ static vbi_bool proxy_client_rpc( vbi_proxy_client * vpc,
    return TRUE;
 
 failure:
-   vbi_asprintf(&vpc->p_errorstr, _("Connection lost due to I/O error."));
+   asprintf(&vpc->p_errorstr, _("Connection lost due to I/O error."));
    return FALSE;
 }
 
@@ -631,7 +633,7 @@ static int proxy_client_read_message( vbi_proxy_client * vpc,
    return ret;
 
 failure:
-   vbi_asprintf(&vpc->p_errorstr, _("Connection lost due to I/O error."));
+   asprintf(&vpc->p_errorstr, _("Connection lost due to I/O error."));
    proxy_client_close(vpc);
    return -1;
 }
@@ -751,7 +753,7 @@ static vbi_bool proxy_client_start_acq( vbi_proxy_client * vpc )
       {
          dprintf1("take_message: CONNECT_CNF: reply version %x, protocol %x\n", p_cnf_msg->magics.protocol_version, p_cnf_msg->magics.protocol_compat_version);
 
-         vbi_asprintf (&vpc->p_errorstr,
+         asprintf (&vpc->p_errorstr,
 		       _("Incompatible server version %u.%u.%u."),
 		       ((p_cnf_msg->magics.protocol_compat_version >> 16) & 0xff),
 		       ((p_cnf_msg->magics.protocol_compat_version >>  8) & 0xff),
@@ -760,7 +762,7 @@ static vbi_bool proxy_client_start_acq( vbi_proxy_client * vpc )
       }
       else if (vpc->endianSwap)
       {  /* endian swapping currently unsupported */
-         vbi_asprintf(&vpc->p_errorstr, _("Incompatible server architecture (endianess mismatch)."));
+         asprintf(&vpc->p_errorstr, _("Incompatible server architecture (endianess mismatch)."));
          goto failure;
       }
       else
@@ -979,6 +981,10 @@ vbi_proxy_client_channel_suspend( vbi_proxy_client * vpc,
                                   VBI_PROXY_SUSPEND cmd )
 {
    /* XXX TODO */
+
+   vpc = vpc;
+   cmd = cmd;
+
    return -1;
 }
 
@@ -1401,6 +1407,8 @@ vbi_proxy_client_flush( vbi_capture * vc )
 static VBI_CAPTURE_FD_FLAGS
 vbi_proxy_client_get_fd_flags(vbi_capture *vc)
 {
+	vc = vc;
+
         return VBI_FD_HAS_SELECT;
 }
 
@@ -1555,7 +1563,7 @@ vbi_proxy_client_create( const char *p_dev_name, const char *p_client_name,
    }
    else
    {
-      vbi_asprintf(pp_errorstr, _("Virtual memory exhausted."));
+      asprintf(pp_errorstr, _("Virtual memory exhausted."));
    }
    return vpc;
 }
@@ -1858,7 +1866,7 @@ vbi_capture_proxy_new( struct vbi_proxy_client *p_proxy_client,
                        char **pp_errorstr )
 {
    pthread_once (&vbi_init_once, vbi_init);
-   vbi_asprintf(pp_errorstr, _("Proxy client interface not compiled."));
+   asprintf(pp_errorstr, _("Proxy client interface not compiled."));
    return NULL;
 }
 
@@ -1906,7 +1914,7 @@ vbi_proxy_client_create(const char *p_dev_name, const char *p_client_name,
                         VBI_PROXY_CLIENT_FLAGS client_flags,
                         char **pp_errorstr, int trace_level)
 {
-   vbi_asprintf(pp_errorstr, _("Proxy client interface not compiled."));
+   asprintf(pp_errorstr, _("Proxy client interface not compiled."));
    return NULL;
 }
 
