@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: teletext.c,v 1.20 2005/01/20 01:40:15 mschimek Exp $ */
+/* $Id: teletext.c,v 1.23 2006/05/24 04:46:45 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -42,32 +42,7 @@
 
 extern const char _zvbi_intl_domainname[];
 
-#ifndef _
-#  ifdef ENABLE_NLS
-#    include <libintl.h>
-#    define _(String) dgettext (_zvbi_intl_domainname, String)
-#    ifdef gettext_noop
-#      define N_(String) gettext_noop (String)
-#    else
-#      define N_(String) (String)
-#    endif
-#  else /* Stubs that do something close enough.  */
-#    define gettext(Msgid) ((const char *) (Msgid))
-#    define dgettext(Domainname, Msgid) ((const char *) (Msgid))
-#    define dcgettext(Domainname, Msgid, Category) ((const char *) (Msgid))
-#    define ngettext(Msgid1, Msgid2, N) \
-       ((N) == 1 ? (const char *) (Msgid1) : (const char *) (Msgid2))
-#    define dngettext(Domainname, Msgid1, Msgid2, N) \
-       ((N) == 1 ? (const char *) (Msgid1) : (const char *) (Msgid2))
-#    define dcngettext(Domainname, Msgid1, Msgid2, N, Category) \
-       ((N) == 1 ? (const char *) (Msgid1) : (const char *) (Msgid2))
-#    define textdomain(Domainname) ((const char *) (Domainname))
-#    define bindtextdomain(Domainname, Dirname) ((const char *) (Dirname))
-#    define bind_textdomain_codeset(Domainname, Codeset) ((const char *) (Codeset))
-#    define _(String) (String)
-#    define N_(String) (String)
-#  endif
-#endif
+#include "intl-priv.h"
 
 #define DEBUG 0
 
@@ -140,7 +115,7 @@ flof_links(vbi_page *pg, vt_page *vtp)
 	for (i = 0; i < COLUMNS + 1; i++) {
 		if (i == COLUMNS || (acp[i].foreground & 7) != col) {
 			for (k = 0; k < 4; k++)
-				if (flof_link_col[k] == col)
+				if ((int) flof_link_col[k] == col)
 					break;
 
 			if (k < 4 && !NO_PAGE(vtp->data.lop.link[k].pgno)) {
@@ -1023,6 +998,8 @@ vbi_page_title(vbi_decoder *vbi, int pgno, int subno, char *buf)
 	ait_entry *ait;
 	int i, j;
 
+	subno = subno;
+
 	if (vbi->vt.top) {
 		for (i = 0; i < 8; i++)
 			if (vbi->vt.btt_link[i].type == 2) {
@@ -1196,6 +1173,7 @@ enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 	struct pex26 *pt, ptmp;
 	int pdc_hr;
 
+	/* XXX nested function not portable, to be removed */
 	void
 	flush(int column)
 	{
@@ -1356,6 +1334,7 @@ enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 		active_column = column;
 	}
 
+	/* XXX nested function not portable, to be removed */
 	void
 	flush_row(void)
 	{
@@ -2086,7 +2065,7 @@ post_enhance(vbi_page *pg, int display_rows)
 	for (row = 0; row <= last_row; row++) {
 		for (column = 0; column < COLUMNS; acp++, column++) {
 			if (1)
-				printv("%c", vbi_printable (acp->unicode));
+				printv("%c", _vbi_to_ascii (acp->unicode));
 			else
 				printv("%04xF%dB%dS%dO%d ", acp->unicode,
 				       acp->foreground, acp->background,
