@@ -18,7 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: glyph.c,v 1.5 2005/01/20 01:40:48 mschimek Exp $ */
+/* $Id: glyph.c,v 1.7 2006/10/06 19:23:55 mschimek Exp $ */
 
 #undef NDEBUG
 
@@ -31,7 +31,6 @@
 
 extern unsigned int	vbi_teletext_unicode(int, int, int);
 extern unsigned int	vbi_teletext_composed_unicode(unsigned int a, unsigned int c);
-extern unsigned int	vbi_caption_unicode(unsigned int c);
 
 static vbi_page *pg;
 static vbi_char ac;
@@ -132,17 +131,23 @@ print_set(const char *name, int s)
 int
 main(int argc, char **argv)
 {
-	int i, j;
+	unsigned int i;
+
+	argc = argc;
+	argv = argv;
 
 	new_page();
 
 	putwstr("ETS 300 706 Table 36: Latin National Option Sub-sets\n\n");
 
 	for (i = 1; i < 14; i++) {
+		unsigned int j;
+
 		for (j = 0; j < sizeof(national) / sizeof(national[0]); j++) {
 			putwchar(vbi_teletext_unicode(1, i, national[j]));
 			putwchar(' ');
 		}
+
 		putwchar('\n');
 	}
 
@@ -165,6 +170,8 @@ main(int argc, char **argv)
 	putwstr("ETS 300 706 Table 47: G1 Block Mosaics Set\n\n");
 
 	for (i = 0; i < 16; i++) {
+		unsigned int j;
+
 		for (j = 2; j < 8; j++) {
 			if (j == 4 || j == 5)
 				putwchar(' ');
@@ -188,13 +195,17 @@ main(int argc, char **argv)
 	putwstr("\n\n");
 
 	for (i = 0; i < 16; i++) {
+		unsigned int j;
+
 		putwchar(vbi_teletext_unicode(2, 0, 0x40 + i));
 		putwstr("  ");
+
 		for (j = 0x40; j < 0x60; j++) {
 			unsigned int c = vbi_teletext_composed_unicode(i, j);
 
 			putwchar((c == 0) ? '-' : c);
 		}
+
 		putwchar('\n');
 	}
 
@@ -209,13 +220,17 @@ main(int argc, char **argv)
 	putwstr("\n\n");
 
 	for (i = 0; i < 16; i++) {
+		unsigned int j;
+
 		putwchar(vbi_teletext_unicode(2, 0, 0x40 + i));
 		putwstr("  ");
+
 		for (j = 0x60; j < 0x80; j++) {
 			unsigned int c = vbi_teletext_composed_unicode(i, j);
 
 			putwchar((c == 0) ? '-' : c);
 		}
+
 		putwchar('\n');
 	}
 
@@ -228,10 +243,14 @@ main(int argc, char **argv)
 	putwstr("EIA 608 Closed Captioning Basic Character Set\n\n");
 
 	for (i = 0; i < 8; i++) {
+		unsigned int j;
+
 		for (j = 0x20; j < 0x80; j += 8) {
-			putwchar(vbi_caption_unicode(j + i));
+			putwchar(vbi_caption_unicode(j + i,
+						     /* to_upper */ FALSE));
 			putwchar(' ');
 		}
+
 		putwchar('\n');
 	}
 
@@ -244,7 +263,8 @@ main(int argc, char **argv)
 	putwstr("EIA 608 Closed Captioning Special Characters\n\n");
 
 	for (i = 0; i < 16; i++) {
-		putwchar(vbi_caption_unicode(i));
+		putwchar(vbi_caption_unicode(0x1130 | i,
+					     /* to_upper */ FALSE));
 	}
 
 	store(17);
