@@ -18,7 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ttxfilter.c,v 1.3 2006/05/22 08:54:36 mschimek Exp $ */
+/* $Id: ttxfilter.c,v 1.6 2006/10/27 04:52:08 mschimek Exp $ */
 
 #undef NDEBUG
 
@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <locale.h>
 #include <assert.h>
 #include <unistd.h>
 #include <sys/time.h>
@@ -145,7 +146,7 @@ teletext			(const uint8_t		buffer[42],
 
 	pmag = vbi_unham16p (buffer);
 	if (pmag < 0) {
-		fprintf (stderr, "Hamming error in pmag\n");
+		fprintf (stderr, "Hamming error in packet number\n");
 		return FALSE;
 	}
 
@@ -294,6 +295,8 @@ old_mainloop			(void)
 						/* n_lines */ 1,
 						timestamp);
 			assert (success);
+
+			fflush (stdout);
 		}
 	}
 
@@ -350,6 +353,8 @@ main				(int			argc,
 	int index;
 	int c;
 
+	setlocale (LC_ALL, "");
+
 	while (-1 != (c = getopt_long (argc, argv, short_options,
 				       long_options, &index))) {
 		switch (c) {
